@@ -142,7 +142,7 @@ get_match_type(const char *str)
 	if (strcmp (str, "contains_outof") == 0)
 		return MATCH_CONTAINS_OUTOF;
 	if (strcmp (str, "int_outof") == 0)
-		return MATCH_CONTAINS_OUTOF;
+		return MATCH_INT_OUTOF;
 	if (strcmp (str, "prefix_outof") == 0)
 		return MATCH_PREFIX_OUTOF;
 	if (strcmp (str, "string_outof") == 0)
@@ -172,7 +172,9 @@ get_merge_type (const char *str)
 	return MERGE_UNKNOWN;
 }
 
-#define ROUND32(len) ((len + 3) & ~0x03)
+#define ROUND(len,align) ((len + align - 1) & -align)
+#define ROUND32(len) ROUND(len, 4)
+#define RULES_ROUND(off) ROUND(off, __alignof__(struct rule))
 
 static void pad32_write(int fd, off_t offset, void *data, size_t len)
 {
@@ -269,9 +271,10 @@ static void store_rule(struct fdi_context *fdi_ctx)
 	if (fdi_ctx->rule.rtype == RULE_UNKNOWN)
 		DIE(("I refuse to store garbage"));
 
-	fdi_ctx->rule.rule_size = sizeof(struct rule) +
-		ROUND32(fdi_ctx->rule.key_len) +
-		ROUND32(fdi_ctx->rule.value_len);
+	fdi_ctx->rule.rule_size =
+	  RULES_ROUND(sizeof(struct rule) +
+		      ROUND32(fdi_ctx->rule.key_len) +
+		      ROUND32(fdi_ctx->rule.value_len));
 
 	pad32_write(fdi_ctx->cache_fd, fdi_ctx->position,
 		&fdi_ctx->rule, sizeof(struct rule));
@@ -297,7 +300,7 @@ static void remember_jump_position(struct fdi_context *fdi_ctx)
 	if (fdi_ctx->depth >= HAL_MAX_INDENT_DEPTH)
 		DIE(("Rule depth overflow"));
 	fdi_ctx->match_at_depth[fdi_ctx->depth++] = fdi_ctx->position;
-};
+}
 
 static void set_jump_position(struct fdi_context *fdi_ctx)
 {
@@ -307,7 +310,7 @@ static void set_jump_position(struct fdi_context *fdi_ctx)
 		DIE(("Rule depth underrun"));
 
 	fdi_ctx->depth--;
-	offset = lseek(fdi_ctx->cache_fd, 0, SEEK_END);
+	offset = RULES_ROUND(lseek(fdi_ctx->cache_fd, 0, SEEK_END));
 	pad32_write(fdi_ctx->cache_fd,
 		fdi_ctx->match_at_depth[fdi_ctx->depth] + offsetof(struct rule, jump_position),
 		&offset, sizeof(fdi_ctx->rule.jump_position));
@@ -331,7 +334,7 @@ start (void *data, const char *el, const char **attr)
 
 	init_rule_struct(&fdi_ctx->rule);
 	fdi_ctx->rule.rtype = get_rule_type(el);
-	fdi_ctx->position = lseek(fdi_ctx->cache_fd, 0, SEEK_END);
+	fdi_ctx->position = RULES_ROUND(lseek(fdi_ctx->cache_fd, 0, SEEK_END));
 	if (fdi_ctx->rule.rtype == RULE_UNKNOWN) return;
 
 	/* get key and attribute for current rule */
@@ -432,6 +435,11 @@ end (void *data, const char *el){
 
 	if (rtype == RULE_UNKNOWN) return;
 	if (rtype == RULE_MATCH){
+		if (fdi_ctx->rule.rtype == RULE_MATCH) {
+			/* the match rule wasn't stored yet, store it now. So it's stored
+			* _before_ jump_position is written into the cache */
+			store_rule(fdi_ctx);
+		}
 		set_jump_position(fdi_ctx);
 		return;
 	}
@@ -510,7 +518,7 @@ rules_add_fdi_file (const char *filename, int fd)
 	/* insert last dummy rule into list */
 	init_rule_struct(&fdi_ctx->rule);
 	fdi_ctx->rule.rtype = RULE_EOF;
-	fdi_ctx->position = lseek(fdi_ctx->cache_fd, 0, SEEK_END);
+	fdi_ctx->position = RULES_ROUND(lseek(fdi_ctx->cache_fd, 0, SEEK_END));
 	store_key(fdi_ctx, filename);
 	store_value(fdi_ctx, "", 0);
 	store_rule(fdi_ctx);
@@ -535,7 +543,7 @@ rules_search_and_add_fdi_files (const char *dir, int fd)
 
 	num_skipped_fdi_files = 0;
 
-	num_entries = scandir (dir, &name_list, 0, _alphasort);
+	num_entries = scandir (dir, &name_list, NULL, _alphasort);
 	if (num_entries == -1) {
 		HAL_ERROR (("Cannot scan '%s': %s", dir, strerror (errno)));
 		goto error;
@@ -599,8 +607,6 @@ error:
 }
 
 
-int haldc_force_recreate = 0;
-
 /* returns number of skipped fdi files or -1 on unrecoverable errors */
 static int
 di_rules_init (void)
@@ -608,7 +614,7 @@ di_rules_init (void)
 	char * cachename;
 	int fd = -1;
 	struct cache_header header;
-	char cachename_temp [PATH_MAX+1];
+	gchar *cachename_temp;
 	char *hal_fdi_source_preprobe = getenv ("HAL_FDI_SOURCE_PREPROBE");
 	char *hal_fdi_source_information = getenv ("HAL_FDI_SOURCE_INFORMATION");
 	char *hal_fdi_source_policy = getenv ("HAL_FDI_SOURCE_POLICY");
@@ -623,8 +629,7 @@ di_rules_init (void)
 	if (haldc_verbose)
 		HAL_INFO (("Loading rules"));
 
-	strncpy(cachename_temp, cachename, PATH_MAX);
-	strncat(cachename_temp, "~", PATH_MAX);
+	cachename_temp = g_strconcat (cachename, "~", NULL);
 
 	fd = open(cachename_temp, O_CREAT|O_RDWR|O_TRUNC, 0644);
 	if(fd < 0) {
@@ -635,7 +640,7 @@ di_rules_init (void)
 	memset(&header, 0, sizeof(struct cache_header));
 	pad32_write(fd, 0, &header, sizeof(struct cache_header));
 
-	header.fdi_rules_preprobe = lseek(fd, 0, SEEK_END);
+	header.fdi_rules_preprobe = RULES_ROUND(lseek(fd, 0, SEEK_END));
 	if (hal_fdi_source_preprobe != NULL) {
 		if ((n = rules_search_and_add_fdi_files (hal_fdi_source_preprobe, fd)) == -1)
 			goto error;
@@ -649,7 +654,7 @@ di_rules_init (void)
 		num_skipped_fdi_files += n;
 	}
 
-	header.fdi_rules_information = lseek(fd, 0, SEEK_END);
+	header.fdi_rules_information = RULES_ROUND(lseek(fd, 0, SEEK_END));
 	if (hal_fdi_source_information != NULL) {
 		if ((n = rules_search_and_add_fdi_files (hal_fdi_source_information, fd)) == -1)
 			goto error;
@@ -663,7 +668,7 @@ di_rules_init (void)
 		num_skipped_fdi_files += n;
 	}
 
-	header.fdi_rules_policy = lseek(fd, 0, SEEK_END);
+	header.fdi_rules_policy = RULES_ROUND(lseek(fd, 0, SEEK_END));
 	if (hal_fdi_source_policy != NULL) {
 		if ((n = rules_search_and_add_fdi_files (hal_fdi_source_policy, fd)) == -1)
 			goto error;
@@ -681,9 +686,8 @@ di_rules_init (void)
 	pad32_write(fd, 0, &header, sizeof(struct cache_header));
 	close(fd);
 	if (rename (cachename_temp, cachename) != 0) {
-		unlink (cachename_temp);
 		HAL_ERROR (("Cannot rename '%s' to '%s': %s", cachename_temp, cachename, strerror (errno)));
-		return -1;
+		goto error;
 	}
 
 	if (haldc_verbose){
@@ -699,9 +703,11 @@ di_rules_init (void)
 	return num_skipped_fdi_files;
 error:
 	HAL_ERROR (("Error generating fdi cache"));
-	if (fd < 0)
+	if (fd >= 0)
 		close (fd);
+
 	unlink (cachename_temp);
+	g_free (cachename_temp);
 	return -1;
 }
 
@@ -712,15 +718,14 @@ error:
  *
  */
 static void
-usage ()
+usage (void)
 {
 	fprintf (stderr, "\n" "usage : hald-generate-fdi-cache [OPTION]\n");
 	fprintf (stderr,
 		 "\n"
-		 "	--force	       Force regeneration of cache.\n"
 		 "	--help		Show this information and exit.\n"
-		 "	--verbose	     Show verbose rule processing output.\n"
-		 "	--version	     Output version information and exit.\n"
+		 "	--verbose	Show verbose rule processing output.\n"
+		 "	--version	Output version information and exit.\n"
 		 "\n"
 		 "hald-generate-fdi-cache is a tool to generate binary cache from FDI files.\n"
 		 "\n"
@@ -741,7 +746,6 @@ int main(int argc, char * argv[])
 		static struct option long_options[] = {
 			{"help", 0, NULL, 0},
 			{"version", 0, NULL, 0},
-			{"force", 0, NULL, 0},
 			{"verbose", 0, NULL, 0},
 			{NULL, 0, NULL, 0}
 		};
@@ -763,8 +767,6 @@ int main(int argc, char * argv[])
 				return 0;
 			} else if (strcmp (opt, "verbose") == 0) {
 				haldc_verbose = 1;
-			} else if (strcmp (opt, "force") == 0) {
-				haldc_force_recreate = 1;
 			}
 			break;
 
