@@ -54,7 +54,7 @@ enum {
 	ACPI_TYPE_BUTTON
 };
 
-#define ACPI_POLL_INTERVAL 30000
+#define ACPI_POLL_INTERVAL 30 /* in seconds */
 
 typedef struct ACPIDevHandler_s
 {
@@ -727,7 +727,7 @@ acpi_synthesize_item (const gchar *fullpath, int acpi_type)
 {
 	HotplugEvent *hotplug_event;
 	HAL_INFO (("Processing %s", fullpath));
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_ACPI;
 	g_strlcpy (hotplug_event->acpi.acpi_path, fullpath, sizeof (hotplug_event->acpi.acpi_path));
@@ -855,7 +855,7 @@ acpi_synthesize_sonypi_display (void)
 	if (!found)
 		return;
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_ACPI;
 	g_strlcpy (hotplug_event->acpi.acpi_path, path, sizeof (hotplug_event->acpi.acpi_path));
@@ -933,13 +933,25 @@ acpi_synthesize_hotplug_events (void)
 	acpi_synthesize_sonypi_display ();
 
 	/* setup timer for things that we need to poll */
-	g_timeout_add (ACPI_POLL_INTERVAL,
+#ifdef HAVE_GLIB_2_14
+	g_timeout_add_seconds (ACPI_POLL_INTERVAL,
+                               acpi_poll,
+                               NULL);
+#else
+	g_timeout_add (1000 * ACPI_POLL_INTERVAL,
 		       acpi_poll,
 		       NULL);
-	/* setup timer for things that we need only to poll infrequently*/
-	g_timeout_add ((ACPI_POLL_INTERVAL*120),
-		       battery_poll_infrequently,
-		       NULL);
+#endif
+
+	/* setup timer for things that we need only to poll infrequently */
+
+        /* don't use g_timeout_add_seconds() here as the related code path
+         * is possibly CPU and time eating and we don't want have any
+         * other timeout synced with this one
+         */
+	g_timeout_add (1000 * ACPI_POLL_INTERVAL * 120,
+                       battery_poll_infrequently,
+                       NULL);
 
 	return TRUE;
 }
@@ -948,13 +960,19 @@ static HalDevice *
 acpi_generic_add (const gchar *acpi_path, HalDevice *parent, ACPIDevHandler *handler)
 {
 	HalDevice *d;
+
+	if (((handler->acpi_type == ACPI_TYPE_BATTERY) || (handler->acpi_type == ACPI_TYPE_AC_ADAPTER)) && _have_sysfs_power_supply)
+		return NULL;
+
 	d = hal_device_new ();
 	hal_device_property_set_string (d, "linux.acpi_path", acpi_path);
 	hal_device_property_set_int (d, "linux.acpi_type", handler->acpi_type);
+
 	if (parent != NULL)
 		hal_device_property_set_string (d, "info.parent", hal_device_get_udi (parent));
 	else
 		hal_device_property_set_string (d, "info.parent", "/org/freedesktop/Hal/devices/computer");
+
 	if (handler->refresh == NULL || !handler->refresh (d, handler, FALSE)) {
 		g_object_unref (d);
 		d = NULL;
@@ -988,11 +1006,10 @@ static gboolean
 acpi_generic_compute_udi (HalDevice *d, ACPIDevHandler *handler)
 {
 	gchar udi[256];
-	hal_util_compute_udi (hald_get_gdl (), udi, sizeof (udi),
-			      "/org/freedesktop/Hal/devices/acpi_%s",
-			      hal_util_get_last_element (hal_device_property_get_string (d, "linux.acpi_path")));
+	hald_compute_udi (udi, sizeof (udi),
+			  "/org/freedesktop/Hal/devices/acpi_%s",
+			  hal_util_get_last_element (hal_device_property_get_string (d, "linux.acpi_path")));
 	hal_device_set_udi (d, udi);
-	hal_device_property_set_string (d, "info.udi", udi);
 	return TRUE;
 }
 
@@ -1253,7 +1270,7 @@ acpi_generate_add_hotplug_event (HalDevice *d)
 	acpi_path = hal_device_property_get_string (d, "linux.acpi_path");
 	acpi_type = hal_device_property_get_int (d, "linux.acpi_type");
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_ADD;
 	hotplug_event->type = HOTPLUG_EVENT_ACPI;
 	g_strlcpy (hotplug_event->acpi.acpi_path, acpi_path, sizeof (hotplug_event->acpi.acpi_path));
@@ -1271,7 +1288,7 @@ acpi_generate_remove_hotplug_event (HalDevice *d)
 	acpi_path = hal_device_property_get_string (d, "linux.acpi_path");
 	acpi_type = hal_device_property_get_int (d, "linux.acpi_type");
 
-	hotplug_event = g_new0 (HotplugEvent, 1);
+	hotplug_event = g_slice_new0 (HotplugEvent);
 	hotplug_event->action = HOTPLUG_ACTION_REMOVE;
 	hotplug_event->type = HOTPLUG_EVENT_ACPI;
 	g_strlcpy (hotplug_event->acpi.acpi_path, acpi_path, sizeof (hotplug_event->acpi.acpi_path));
