@@ -29,13 +29,18 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#include <linux/input.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#ifdef HAL_LINUX_INPUT_HEADER_H
+  #include HAL_LINUX_INPUT_HEADER_H
+#else
+  #include <linux/input.h>
+#endif
+
 #include "libhal/libhal.h"
 #include "../../logger.h"
 
@@ -76,6 +81,10 @@ main (int argc, char *argv[])
 		sw = SW_TABLET_MODE;
 	else if (strcmp (button_type, "headphone_insert") == 0)
 		sw = SW_HEADPHONE_INSERT;
+#ifdef SW_RADIO
+	else if (strcmp (button_type, "radio") == 0)
+		sw = SW_RADIO;
+#endif
 	else
 		goto out;
 
