@@ -179,6 +179,7 @@ hal_util_get_int_from_file (const gchar *directory, const gchar *file, gint *res
 	char buf[64];
 	gchar path[HAL_PATH_MAX];
 	gboolean ret;
+	gint _result;
 
 	f = NULL;
 	ret = FALSE;
@@ -187,18 +188,21 @@ hal_util_get_int_from_file (const gchar *directory, const gchar *file, gint *res
 
 	f = fopen (path, "rb");
 	if (f == NULL) {
-		HAL_ERROR (("Cannot open '%s'", path));
+		//HAL_ERROR (("Cannot open '%s'", path));
 		goto out;
 	}
 
 	if (fgets (buf, sizeof (buf), f) == NULL) {
-		HAL_ERROR (("Cannot read from '%s'", path));
+		//HAL_ERROR (("Cannot read from '%s'", path));
 		goto out;
 	}
 
-	/* TODO: handle error condition */
-	*result = strtol (buf, NULL, base);
-	ret = TRUE;
+	errno = 0;
+	_result = strtol (buf, NULL, base);
+	if (errno == 0) {
+		ret = TRUE;
+		*result = _result;
+	}
 
 out:
 	if (f != NULL)
@@ -229,6 +233,7 @@ hal_util_get_uint64_from_file (const gchar *directory, const gchar *file, guint6
 	char buf[64];
 	gchar path[HAL_PATH_MAX];
 	gboolean ret;
+	guint64 _result;
 
 	f = NULL;
 	ret = FALSE;
@@ -237,19 +242,21 @@ hal_util_get_uint64_from_file (const gchar *directory, const gchar *file, guint6
 
 	f = fopen (path, "rb");
 	if (f == NULL) {
-		HAL_ERROR (("Cannot open '%s'", path));
+		//HAL_ERROR (("Cannot open '%s'", path));
 		goto out;
 	}
 
 	if (fgets (buf, sizeof (buf), f) == NULL) {
-		HAL_ERROR (("Cannot read from '%s'", path));
+		//HAL_ERROR (("Cannot read from '%s'", path));
 		goto out;
 	}
 
-	/* TODO: handle error condition */
-	*result = strtoll (buf, NULL, base);
-
-	ret = TRUE;
+	errno = 0;
+	_result = strtoll (buf, NULL, base);
+	if (errno == 0) {
+		ret = TRUE;
+		*result = _result;
+	}
 
 out:
 	if (f != NULL)
@@ -272,96 +279,6 @@ hal_util_set_uint64_from_file (HalDevice *d, const gchar *key, const gchar *dire
 	return ret;
 }
 
-gboolean
-hal_util_get_bcd2_from_file (const gchar *directory, const gchar *file, gint *result)
-{
-	FILE *f;
-	char buf[64];
-	gchar path[HAL_PATH_MAX];
-	gboolean ret;
-	gint digit;
-	gint left, right;
-	gboolean passed_white_space;
-	gint num_prec;
-	gsize len;
-	gchar c;
-	guint i;
-
-	f = NULL;
-	ret = FALSE;
-
-	g_snprintf (path, sizeof (path), "%s/%s", directory, file);
-
-	f = fopen (path, "rb");
-	if (f == NULL) {
-		HAL_ERROR (("Cannot open '%s'", path));
-		goto out;
-	}
-
-	if (fgets (buf, sizeof (buf), f) == NULL) {
-		HAL_ERROR (("Cannot read from '%s'", path));
-		goto out;
-	}
-
-	left = 0;
-	len = strlen (buf);
-	passed_white_space = FALSE;
-	for (i = 0; i < len && buf[i] != '.'; i++) {
-		if (g_ascii_isspace (buf[i])) {
-			if (passed_white_space)
-				break;
-			else
-				continue;
-		}
-		passed_white_space = TRUE;
-		left *= 16;
-		c = buf[i];
-		digit = (int) (c - '0');
-		left += digit;
-	}
-	i++;
-	right = 0;
-	num_prec = 0;
-	for (; i < len; i++) {
-		if (g_ascii_isspace (buf[i]))
-			break;
-		if (num_prec == 2)	        /* Only care about two digits 
-						 * of precision */
-			break;
-		right *= 16;
-		c = buf[i];
-		digit = (int) (c - '0');
-		right += digit;
-		num_prec++;
-	}
-
-	for (; num_prec < 2; num_prec++)
-		right *= 16;
-
-	*result = left * 256 + (right & 255);
-	ret = TRUE;
-
-out:
-	if (f != NULL)
-		fclose (f);
-
-	return ret;
-}
-
-gboolean
-hal_util_set_bcd2_from_file (HalDevice *d, const gchar *key, const gchar *directory, const gchar *file)
-{
-	gint value;
-	gboolean ret;
-
-	ret = FALSE;
-
-	if (hal_util_get_bcd2_from_file (directory, file, &value))
-		ret = hal_device_property_set_int (d, key, value);
-
-	return ret;
-}
-
 gchar *
 hal_util_get_string_from_file (const gchar *directory, const gchar *file)
 {
@@ -379,13 +296,13 @@ hal_util_get_string_from_file (const gchar *directory, const gchar *file)
 
 	f = fopen (path, "rb");
 	if (f == NULL) {
-		HAL_ERROR (("Cannot open '%s'", path));
+		//HAL_ERROR (("Cannot open '%s'", path));
 		goto out;
 	}
 
 	buf[0] = '\0';
 	if (fgets (buf, sizeof (buf), f) == NULL) {
-		HAL_ERROR (("Cannot read from '%s'", path));
+		//HAL_ERROR (("Cannot read from '%s'", path));
 		goto out;
 	}
        
@@ -451,6 +368,7 @@ hal_util_set_double_from_file (HalDevice *d, const gchar *key, const gchar *dire
 	ret = FALSE;
 
 	if ((buf = hal_util_get_string_from_file (directory, file)) != NULL) {
+		errno = 0;
 		value = strtod(buf, &end);
 		if (errno != ERANGE) {
 			ret = hal_device_property_set_double (d, key, value);
@@ -461,36 +379,49 @@ hal_util_set_double_from_file (HalDevice *d, const gchar *key, const gchar *dire
 }
 
 void
-hal_util_compute_udi (HalDeviceStore *store, gchar *dst, gsize dstsize, const gchar *format, ...)
+hal_util_make_udi_unique (HalDeviceStore *store, gchar *udi, gsize udisize, const char *original_udi)
 {
-	guint i;
-	va_list args;
-	gchar buf[256];
+	int i;
 
-	va_start (args, format);
-	g_vsnprintf (buf, sizeof (buf), format, args);
-	va_end (args);
-
-	g_strcanon (buf, 
-		    "/_"
-		    "abcdefghijklmnopqrstuvwxyz"
-		    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
-		    "1234567890", '_');
-
-	g_strlcpy (dst, buf, dstsize);
-	if (hal_device_store_find (store, dst) == NULL)
+	if (hal_device_store_find (store, original_udi) == NULL) {
+		g_strlcpy (udi, original_udi, udisize);
 		goto out;
+	}
 
 	for (i = 0; ; i++) {
-		g_snprintf (dst, dstsize, "%s_%d", buf, i);
-		if (hal_device_store_find (store, dst) == NULL)
+		g_snprintf (udi, udisize, "%s_%d", original_udi, i);
+		if (hal_device_store_find (store, udi) == NULL) {
 			goto out;
+		}
 	}
 
 out:
 	;
 }
 
+void
+hal_util_compute_udi_valist (HalDeviceStore *store, gchar *dst, gsize dstsize, const gchar *format, va_list args)
+{
+	g_vsnprintf (dst, dstsize, format, args);
+
+	g_strcanon (dst,
+		    "/_"
+		    "abcdefghijklmnopqrstuvwxyz"
+		    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+		    "1234567890", '_');
+}
+
+
+void
+hal_util_compute_udi (HalDeviceStore *store, gchar *dst, gsize dstsize, const gchar *format, ...)
+{
+	va_list args;
+
+	va_start (args, format);
+	hal_util_compute_udi_valist (store, dst, dstsize, format, args);
+	va_end (args);
+}
+
 
 gboolean
 hal_util_path_ascend (gchar *path)
@@ -1125,7 +1056,7 @@ hal_util_hexdump (const void *mem, unsigned int size)
 	const char *buf = (const char *) mem;
 
 	n = 0;
-	printf ("Dumping %d=0x%x bytes\n", size, size);
+	printf ("Dumping %u=0x%x bytes\n", size, size);
 	while (n < size) {
 
 		printf ("0x%04x: ", n);
